@@ -131,7 +131,9 @@ class AbstractPutTests:
 
             # Without recursive does nothing
             fs.put(s, t)
-            assert fs.ls(target) == ([] if supports_empty_directories else [dummy])
+            assert fs.ls(target, detail=False) == (
+                [] if supports_empty_directories else [dummy]
+            )
 
             # With recursive
             fs.put(s, t, recursive=True)
@@ -158,7 +160,9 @@ class AbstractPutTests:
                 assert fs.isfile(fs_join(target, "subdir", "nesteddir", "nestedfile"))
 
                 fs.rm(fs_join(target, "subdir"), recursive=True)
-            assert fs.ls(target) == ([] if supports_empty_directories else [dummy])
+            assert fs.ls(target, detail=False) == (
+                [] if supports_empty_directories else [dummy]
+            )
 
             # Limit recursive by maxdepth
             fs.put(s, t, recursive=True, maxdepth=1)
@@ -182,7 +186,9 @@ class AbstractPutTests:
                 assert not fs.exists(fs_join(target, "subdir", "nesteddir"))
 
                 fs.rm(fs_join(target, "subdir"), recursive=True)
-            assert fs.ls(target) == ([] if supports_empty_directories else [dummy])
+            assert fs.ls(target, detail=False) == (
+                [] if supports_empty_directories else [dummy]
+            )
 
     def test_put_directory_to_new_directory(
         self,
@@ -275,7 +281,9 @@ class AbstractPutTests:
                 ],
                 recursive=True,
             )
-            assert fs.ls(target) == ([] if supports_empty_directories else [dummy])
+            assert fs.ls(target, detail=False) == (
+                [] if supports_empty_directories else [dummy]
+            )
 
             # With recursive
             for glob, recursive in zip(["*", "**"], [True, False]):
@@ -294,7 +302,9 @@ class AbstractPutTests:
                     ],
                     recursive=True,
                 )
-                assert fs.ls(target) == ([] if supports_empty_directories else [dummy])
+                assert fs.ls(target, detail=False) == (
+                    [] if supports_empty_directories else [dummy]
+                )
 
                 # Limit recursive by maxdepth
                 fs.put(
@@ -315,7 +325,9 @@ class AbstractPutTests:
                     ],
                     recursive=True,
                 )
-                assert fs.ls(target) == ([] if supports_empty_directories else [dummy])
+                assert fs.ls(target, detail=False) == (
+                    [] if supports_empty_directories else [dummy]
+                )
 
     def test_put_glob_to_new_directory(
         self, fs, fs_join, fs_target, local_join, local_bulk_operations_scenario_0
@@ -463,7 +475,9 @@ class AbstractPutTests:
                 ],
                 recursive=True,
             )
-            assert fs.ls(target) == ([] if supports_empty_directories else [dummy])
+            assert fs.ls(target, detail=False) == (
+                [] if supports_empty_directories else [dummy]
+            )
 
     def test_put_list_of_files_to_new_directory(
         self, fs, fs_join, fs_target, local_join, local_bulk_operations_scenario_0
